package com.cecf.observe;

import io.opentelemetry.proto.common.v1.AnyValue;
import io.opentelemetry.proto.common.v1.ArrayValue;
import io.opentelemetry.proto.common.v1.KeyValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class KVList {
    Logger logger = LoggerFactory.getLogger(KVList.class);
    Map<String, AnyValue> m;

    public KVList(List<KeyValue> kvs) {
        this.m = new HashMap<>();
        for (KeyValue o : kvs) {
            this.m.put(o.getKey(), o.getValue());
        }
    }

    public String getString(String k) {
        return getString(k, "");
    }
    public String getString(String k, String de) {
        AnyValue v = this.m.get(k);
        if (v == null) {
            return de;
        }
        return v.getStringValue();
    }

    String anyValueToString(AnyValue v) {
        if (v.hasBoolValue()) {
            return v.getBoolValue() ? "true" : "false";
        }
        if (v.hasDoubleValue()) {
            return Double.toString(v.getDoubleValue());
        }
        if (v.hasIntValue()) {
            return Long.toString(v.getIntValue());
        }
        if (v.hasBytesValue()) {
            return v.getBytesValue().toString();
        }
        if (v.hasKvlistValue()) {
            return v.getKvlistValue().toString();
        }
        return v.getStringValue();
    }

    public Map<String, String> toStringMap() {
        Map<String, String> strMap = new HashMap<>();
        for (Map.Entry<String, AnyValue> en : m.entrySet()) {
            String key = en.getKey();
            AnyValue v = en.getValue();
            if (v.hasArrayValue()) {
                List<String> valueStrList = new ArrayList<>();
                ArrayValue arrayValueValue = v.getArrayValue();
                for (int i = 0; i < arrayValueValue.getValuesCount(); i++) {
                    AnyValue valueItem = arrayValueValue.getValues(i);
                    valueStrList.add(anyValueToString(valueItem));
                }
                strMap.put(key, String.join(";", valueStrList));
                continue;
            }
            strMap.put(key, anyValueToString(v));
        }
        return strMap;
    }

    public long getLong(String k) {
        AnyValue v = this.m.get(k);
        if (v == null) {
            return 0;
        }
        return v.getIntValue();
    }
}

