package com.cecf.observe;

import java.util.*;

import static com.cecf.observe.OtelUtils.StatusOK;

public class PojoSpan {
    public PojoSpan() {
        SpanAttributes = new HashMap<>();
        ResourceAttributes = new HashMap<>();
        RPCRequest = new HashMap<>();
        Events = new ArrayList<>();
        Links = new ArrayList<>();
        Exceptions = new ArrayList<>();
        AppName = "UNSET";
    }

    public enum SpanScopeType {
        SPAN_SCOPE_TYPE_HTTP,
        SPAN_SCOPE_TYPE_RPC,

        SPAN_SCOPE_TYPE_DB,
        SPAN_SCOPE_TYPE_MQ,
        SPAN_SCOPE_TYPE_GENERIC;

    }

    //TODO:
    public SpanScopeType getSpanScopeType() {
        if (isHttpScopeType()) {
            return SpanScopeType.SPAN_SCOPE_TYPE_HTTP;
        }
        if (isDBScopeType()) {
            return SpanScopeType.SPAN_SCOPE_TYPE_DB;
        }
        if (isRPCScopeType()) {
            return SpanScopeType.SPAN_SCOPE_TYPE_RPC;
        }
        if (isMQScopeType()) {
            return SpanScopeType.SPAN_SCOPE_TYPE_MQ;
        }
        return SpanScopeType.SPAN_SCOPE_TYPE_GENERIC;
    }

    public boolean isMQScopeType() {
        return hasAttributes("messaging.system");
    }

    public boolean isRPCScopeType() {
        return hasAttributes("rpc.system");
    }

    public boolean isDBScopeType() {
        return hasAttributes("db.system");
    }

    public boolean isHttpScopeType() {
        return hasAttributes("http.request.method");
    }

    public boolean hasAttributes(String key) {
        return SpanAttributes.containsKey(key) || ResourceAttributes.containsKey(key);
    }


    public String getAttribute(String de, String... keys) {
        for (String key : keys) {
            if (SpanAttributes.containsKey(key)) {
                return SpanAttributes.get(key);
            }
            if (ResourceAttributes.containsKey(key)) {
                return ResourceAttributes.get(key);
            }
        }
        return de;
    }

    public boolean hasAppName() {
        return AppName != null && !AppName.equals("UNSET") && !AppName.isEmpty();
    }

    public Date Timestamp;
    public String TraceID;
    public String SpanID;
    public String ParentSpanID;
    public String TraceState;
    public String SpanName;
    public int SpanKind;
    public String ScopeName;
    public String ScopeVersion;
    public Map<String, String> SpanAttributes;
    public String ServiceName;
    public Map<String, String> ResourceAttributes;
    public String AppName;
    public long Duration;
    public String ContainerID;
    public String SrcIP;
    public int SrcPort;
    public String TargetIP;
    public long TargetPort;

    public int HttpCode;
    public String HttpMethod;
    public String HttpURL;
    public String RPCType;
    public String RPCName;
    public Map<String, String> RPCRequest;
    public long RPCResult;

    public String FuncNamespace;
    public String FuncName;
    public long FuncLineNO;
    public int FuncResult;
    public int StatusCode;
    public String StatusMessage;

    public List<TraceEvent> Events;
    public List<TraceLink> Links;
    public String DBStatement;
    public String DBConnectionString;
    public List<TraceException> Exceptions;

    public String getServiceInstanceID() {
        String podUid = this.ResourceAttributes.get("k8s.pod.uid");
        if (podUid != null) {
            return podUid;
        }
        return this.ResourceAttributes.getOrDefault("k8s.pod.ip", "unknown");
    }

    public long getEndTimeUnixMill() {
        return Timestamp.getTime() + (long) Math.floor(Duration / 1e6);
    }

    public boolean isSuccess() {
        return this.StatusCode == StatusOK;
    }

    public String getStatusCode() {
        if (isHttpScopeType()) {
            return getAttribute("200", "http.response.status_code");
        }
        return "200";
    }

    public boolean hasExceptionEvents() {
        return Exceptions != null && !Exceptions.isEmpty();
    }

    public boolean HasEvent() {
        if (hasExceptionEvents()) {
            return true;
        }
        return Events != null && !Events.isEmpty();
    }
}


